with Ada.Text_IO;
with Game.Time;
with Game.Screen;
with Game.Input;
with Implementation;
procedure Akanvader is
	pragma Linker_Options("-s");
	use Implementation;
	Actors: Actor_List.List;
	Fighter: Fighter_Access;
	Game_Screen: Game.Screen.Handle;
	Back_Buffer: Game.Screen.Screen_Buffer;
	Paused: Boolean;
begin
	Ada.Text_IO.Put("Akanvader...");

	Game.Screen.Set_Title("Akanvader");
	Game.Screen.Set_Window_Size_80x25;

	Game_Screen := Game.Screen.New_Screen;

	Paused := False;

	Fighter := new Implementation.Fighter;
	Actor_List.Append(Actors, Actor_Access(Fighter));

	for Y in 0..5 loop
		for X in 0..7 loop
			declare
				Enemy: Enemy_Access := new Implementation.Enemy;
			begin
				Enemy.Location.X := X * 4;
				Enemy.Location.Y := Y * 2;
				Enemy.Way := Right;
				Enemy.Way_Length := 80 - 8 * 4 + 2;
				Enemy.Way_Step := Enemy.Way_Length;
				Actor_List.Append(Actors, Actor_Access(Enemy));
			end;
		end loop;
	end loop;
	
	for X in 0..3 loop
		declare
			BX: constant := (20 - 6) / 2;
		begin
			declare
				S: Shield_Access := new Shield;
			begin
				S.Location := (X => X * 20 + BX, Y => 19);
				Actor_List.Append(Actors, Actor_Access(S));
			end;
			declare
				S: Shield_Access := new Shield;
			begin
				S.Location := (X => X * 20 + BX, Y => 20);
				Actor_List.Append(Actors, Actor_Access(S));
			end;
			declare
				S: Shield_Access := new Shield;
			begin
				S.Location := (X => X * 20 + BX + 2, Y => 19);
				Actor_List.Append(Actors, Actor_Access(S));
			end;
			declare
				S: Shield_Access := new Shield;
			begin
				S.Location := (X => X * 20 + BX + 4, Y => 19);
				Actor_List.Append(Actors, Actor_Access(S));
			end;
			declare
				S: Shield_Access := new Shield;
			begin
				S.Location := (X => X * 20 + BX + 4, Y => 20);
				Actor_List.Append(Actors, Actor_Access(S));
			end;
		end;
	end loop;
	
	delay 0.5;
	Ada.Text_IO.Put("Setup!!");
	Ada.Text_IO.New_Line;
	delay 0.5;

	Game.Screen.Present(Game_Screen);
	Game.Time.Reset;
	Game_Loop: loop
		if Game.Screen.Has_Events(Game.Screen.Input) then
			declare
				Message : Game.Screen.Event;
			begin
				Message := Game.Screen.Get_Event(Game.Screen.Input);
				if Game.Screen.Is_Lost_Focus(Message) then
					Paused := True;
				elsif Game.Screen.Is_Escape_Press(Message) then
					Paused := not Paused;
					Game.Time.Reset;
				end if;
			end;
		elsif not Paused then
			declare
				State: Game.Input.State := Game.Input.Get_State;
			begin
				if State(Game.Input.Left) then
					Move(Fighter.all, Left);
					Test_Hit(Actors, Actor_Access(Fighter));
				end if;
				if State(Game.Input.Right) then
					Move(Fighter.all, Right);
					Test_Hit(Actors, Actor_Access(Fighter));
				end if;
				if State(Game.Input.Shoot) then
					if Fighter.Shooting_Interval = 0 then
						declare
							Shot: Shot_Access := new Fighter_Shot;
						begin
							Shot.Location := Shooting_Location(Fighter.all);
							Actor_List.Append(Actors, Actor_Access(Shot));
							Fighter.Shooting_Interval := 30;
						end;
					end if;
				else
					Fighter.Shooting_Interval := 0;
				end if;
			end;
			declare
				use Actor_List;
				I: Cursor := First(Actors);
			begin
				while Has_Element(I) loop
					Advance(Element(I), Actors);
					I := Next(I);
				end loop;
			end;
			Game.Screen.Clear(Back_Buffer);
			declare
				use Actor_List;
				I, Next: Cursor := First(Actors);
				Actor: Actor_Access;
				Base: Base_Access;
			begin
				while Has_Element(I) loop
					Next := Actor_List.Next(I);
					Actor := Element(I);
					if Actor.Deleted then
						if Actor.all in Implementation.Fighter'Class then
							exit Game_Loop;
						else
							Delete(Actors, I);
							Base := Base_Access(Actor);
							Actor := null;
							Dispose(Base);
						end if;
					else
						Draw(Actor.all, Back_Buffer);
					end if;
					I := Next;
				end loop;
			end;
			Game.Time.Wait;
			Game.Screen.Write_All(Game_Screen, Back_Buffer);
		end if;
	end loop Game_Loop;

	Game.Screen.Present(Game.Screen.Output);
	Game.Screen.Dispose(Game_Screen);
	
	Ada.Text_IO.Put("Game Over!!");
	Ada.Text_IO.New_Line;
end Akanvader;
