package body Game.Screen is

	procedure Set_Title(Text: in String) is
	begin
		Windows.SetConsoleTitle(Windows.Addr(Text & Windows.Nul));
	end;

	procedure Set_Window_Size_80x25 is
		Window_Rect: aliased constant Windows.SMALL_RECT := (0, 0, 79, 24);
	begin
		Windows.SetConsoleWindowInfo(Output, Windows.BOOL_TRUE, Window_Rect'Unchecked_Access);
	end;

	function New_Screen return Windows.HANDLE is
		use type Windows.COORD;
		use type Windows.DWORD;
		Cursor_Info: aliased constant Windows.CONSOLE_CURSOR_INFO := (Size => 100, Visible => Windows.BOOL_FALSE);
		Result: Windows.HANDLE;
		Size: aliased constant Windows.COORD := (X => 80, Y => 25);
	begin
		Result := Windows.CreateConsoleScreenBuffer(Windows.GENERIC_READ or Windows.GENERIC_WRITE, 
			0, null, Windows.CONSOLE_TEXTMODE_BUFFER, System.Null_Address);
		Windows.SetConsoleCursorInfo(Result, Cursor_Info'Unchecked_Access);
		Windows.SetConsoleScreenBufferSize(Result, +Size);
		return Result;
	end;

	procedure Present(Screen: Handle) is
	begin
		Windows.SetConsoleActiveScreenBuffer(Screen);
	end;

	procedure Dispose(Screen: Handle) is
	begin
		Windows.CloseHandle(Screen);
	end;

	procedure Clear(Screen: Windows.HANDLE) is
		use type Windows.COORD;
		Size: constant Windows.COORD := (X => 80, Y => 25);
		Start: constant Windows.COORD := (X => 0, Y => 0);
		Image: array(0 .. 80 * 25 - 1) of aliased Windows.CHAR_INFO;
		R: aliased Windows.SMALL_RECT;
	begin
		for I in Image'Range loop
			Image(I).Char.AsciiChar := Windows.AChar'Val(32);
			Image(I).Attributes := 0;
		end loop;
		R.Left   :=  0;
		R.Top    :=  0;
		R.Right  := 79;
		R.Bottom := 24;
		Windows.WriteConsoleOutput(Screen, Image(0)'Unchecked_Access, +Size, +Start, R'Unchecked_Access);
	end;

	procedure Write(Screen: Windows.HANDLE; X, Y: Integer; Text: in String; Attribute: Windows.WORD) is
		use type Windows.UINT;
		use type Windows.COORD;
		L: Integer := Integer(Windows.UINT(Text'Length) and 16#ff#);
		Buffer: aliased array(0 .. 16#ff#) of aliased Windows.CHAR_INFO;
		Size: aliased Windows.COORD;
		R: aliased Windows.SMALL_RECT;
		Zero: constant Windows.COORD := (X | Y => 0);
	begin
		for I in 0 .. L - 1 loop
			Buffer(I).Char.AsciiChar := Windows.AChar(Text(Text'First + I));
			Buffer(I).Attributes := Attribute;
		end loop;

		Size.X := Windows.SHORT(L);
		Size.Y := 1;

		R.Left   := Windows.SHORT(X);
		R.Top    := Windows.SHORT(Y);
		R.Right  := Windows.SHORT(X + L - 1);
		R.Bottom := Windows.SHORT(Y);

		Windows.WriteConsoleOutput(Screen, Buffer(0)'Unchecked_Access, +Size, +Zero, R'Unchecked_Access);
	end;

	procedure Clear(Buffer: in out Screen_Buffer) is
		White_Space: constant Windows.CHAR_INFO := (
			Char => (Kind => Windows.Ascii, AsciiChar => ' '), Attributes => 0);
	begin
		for Y in Buffer'Range(1) loop
			for X in Buffer'Range(2) loop
				Buffer(Y, X) := White_Space;
			end loop;
		end loop;
	end;
	
	procedure Write(Buffer: in out Screen_Buffer; X, Y: Integer; Text: in String; Attribute: Windows.WORD) is
		use type Windows.BOOL;
		type T is (Single, Lead, Trail);
		Pos: T;
	begin
		if Y in Buffer'Range(1) then
			Pos := Single;
			for I in Buffer'Range(2) loop
				if Pos = Lead then
					Pos := Trail;
				elsif +Windows.IsDBCSLeadByte(Buffer(Y, I).Char.AsciiChar) then
					Pos := Lead;
				else
					Pos := Single;
				end if;
				if I in X .. X + Text'Length - 1 then
					Buffer(Y, I) := (
						Char       => (
							Kind      => Windows.Ascii, 
							AsciiChar => Windows.AChar(Text(I - X + Text'First))), 
						Attributes => Attribute);
					if I = X and Pos = Trail then
						Buffer(Y, I - 1).Char.AsciiChar := ' ';
					elsif I = X + Text'Length - 1 and Pos = Lead then
						Buffer(Y, I + 1).Char.AsciiChar := ' ';
					end if;
				end if;
			end loop;
		end if;
	end;
	
	procedure Write_All(Screen: Handle; Buffer: Screen_Buffer) is
		use type Windows.COORD;
		Size: constant Windows.COORD := (X => 80, Y => 25);
		Start: constant Windows.COORD := (X => 0, Y => 0);
		R: aliased Windows.SMALL_RECT := (Left => 0, Top => 0, Right => 79, Bottom => 24);
	begin
		Windows.WriteConsoleOutput(Screen, Buffer(0, 0)'Unchecked_Access, +Size, +Start, R'Unchecked_Access);
	end;

	function Has_Events(Input: HANDLE) return Boolean is
		use type Windows.BOOL;
		use type Windows.DWORD;
		Count: aliased Windows.DWORD;
	begin
		return +Windows.GetNumberOfConsoleInputEvents(Input, Count'Unchecked_Access) and then Count > 0;
	end;

	function Get_Event(Input: HANDLE) return Windows.INPUT_RECORD is
		Dummy: Windows.BOOL;
		Result: aliased Windows.INPUT_RECORD;
		Read: aliased Windows.DWORD;
	begin
		Dummy := Windows.ReadConsoleInput(Input, Result'Unchecked_Access, 1, Read'Unchecked_Access);
		return Result;
	end;

	function Is_Lost_Focus(Message: in Event) return Boolean is
		use type Windows.BOOL;
		use type Windows.WORD;
	begin
		return Message.EventType = Windows.FOCUS_EVENT and then
			not Message.Event.FocusEvent.SetFocus;
	end;
	
	function Is_Escape_Press(Message: in Event) return Boolean is
		use type Windows.WORD;
		use type Windows.BOOL;
	begin
		return Message.EventType = Windows.KEY_EVENT and then
			+Message.Event.KeyEvent.KeyDown and then
			Message.Event.KeyEvent.VirtualKeyCode = Windows.VK_ESCAPE;
	end;

end Game.Screen;
