package body Implementation is

	procedure Test_Hit(List: in out Actor_List.List; It: access Actor'Class) is
		use Actor_List;
		I: Cursor;
		Other: Actor_Access;
	begin
		I := First(List);
		Enum_Loop: while Has_Element(I) loop
			Other := Element(I);
			if not Other.Deleted and Other /= Actor_Access(It) then
				if Other.Location.X + Other.Width > It.Location.X and
					Other.Location.X < It.Location.X + It.Width and
					Other.Location.Y + Other.Height > It.Location.Y and
					Other.Location.Y < It.Location.Y + It.Height 
				then
					Collide(It.all, Other);
					Collide(Other.all, Actor_Access(It));
					if It.Deleted then
						exit Enum_Loop;
					end if;
				end if;
			end if;
			I := Next(I);
		end loop Enum_Loop;
	end Test_Hit;

	-- Fighter

	procedure Initialize(Object: in out Fighter) is
	begin
		Object.Location.X := 39;
		Object.Location.Y := 22;
		Object.Width := 2;
		Object.Height := 1;
		Object.Shooting_Interval := 0;
	end;
	
	procedure Advance(Object: access Fighter; List: in out Actor_List.List) is
	begin
		if Object.Shooting_Interval > 0 then
			Object.Shooting_Interval := Object.Shooting_Interval - 1;
		end if;
	end;

	procedure Draw(Object: in out Fighter; Screen: in out Game.Screen.Screen_Buffer) is
	begin
		Game.Screen.Write(Screen, Object.Location.X, Object.Location.Y, "", 9);
	end;

	procedure Collide(Object: in out Fighter; Other: Actor_Access) is
	begin
		if (Other.all in Enemy'Class) or (Other.all in Enemy_Shot'Class) then
			Object.Deleted := True;
		end if;
	end;

	procedure Move(Object: in out Fighter; Way: Direction) is
	begin
		case Way is
		when Left =>
			if Object.Location.X > 0 then
				Object.Location.X := Object.Location.X - 1;
			end if;
		when Right =>
			if Object.Location.X < 78 then
				Object.Location.X := Object.Location.X + 1;
			end if;
		end case;
		Object.Last_Moved := Way;
	end;
	
	function Shooting_Location(Object: in Fighter) return Point is
		Result: Point;
	begin
		Result.X := Object.Location.X;
		if Object.Last_Moved = Left then
			Result.X := Result.X + 1;
		end if;
		Result.Y := Object.Location.Y;
		return Result;
	end;

	-- Enemy

	procedure Initialize(Object: in out Enemy) is
	begin
		Object.Width := 2;
		Object.Height := 1;
		Object.Interval := 0;
	end;
	
	procedure Advance(Object: access Enemy; List: in out Actor_List.List) is
	begin
		if Object.Interval > 0 then
			Object.Interval := Object.Interval - 1;
		else
			Object.Interval := 4;
			
			if Object.Way_Step = 0 then
				Object.Way_Step := Object.Way_Length;
				if Object.Way = Left then 
					Object.Way := Right;
				else
					Object.Way := Left;
				end if;
				Object.Location.Y := Object.Location.Y + 1;
			else
				Object.Way_Step := Object.Way_Step - 1;
				case Object.Way is
				when Left =>
					Object.Location.X := Object.Location.X - 1;
				when Right =>
					Object.Location.X := Object.Location.X + 1;
				end case;
			end if;
			
			Test_Hit(List, Object);
		end if;
		
		if Random.Random(Seed) = 0 then
			declare
				S: Shot_Access := new Enemy_Shot;
			begin
				S.Location := Object.Location;
				if Object.Way = Left then
					S.Location.X := S.Location.X + 1;
				end if;
				Actor_List.Append(List, Actor_Access(S));
			end;
		end if;
	end;
	
	procedure Draw(Object: in out Enemy; Screen: in out Game.Screen.Screen_Buffer) is
	begin
		Game.Screen.Write(Screen, Object.Location.X, Object.Location.Y, "", 10);
	end;

	procedure Collide(Object: in out Enemy; Other: Actor_Access) is
	begin
		if Other.all in Fighter_Shot'Class then
			Object.Deleted := True;
		end if;
	end;
	
	-- Shot

	procedure Initialize(Object: in out Shot) is
	begin
		Object.Width := 1;
		Object.Height := 1;
		Object.Interval := 2;
	end;
	
	procedure Draw(Object: in out Shot; Screen: in out Game.Screen.Screen_Buffer) is
	begin
		Game.Screen.Write(Screen, Object.Location.X, Object.Location.Y, "|", 15);
	end;

	procedure Collide(Object: in out Shot; Other: Actor_Access) is
	begin
		if Other.all not in Shot'Class then
			Object.Deleted := True;
		end if;
	end;

	-- Fighter_Shot

	procedure Advance(Object: access Fighter_Shot; List: in out Actor_List.List) is
	begin
		if Object.Interval > 0 then
			Object.Interval := Object.Interval - 1;
		else
			Object.Interval := 2;
			Object.Location.Y := Object.Location.Y - 1;
			if Object.Location.Y < 0 then
				Object.Deleted := True;
			end if;
			Test_Hit(List, Object);
		end if;
	end;
	
	-- Enemy_Shot

	procedure Advance(Object: access Enemy_Shot; List: in out Actor_List.List) is
	begin
		if Object.Interval > 0 then
			Object.Interval := Object.Interval - 1;
		else
			Object.Interval := 2;
			Object.Location.Y := Object.Location.Y + 1;
			if Object.Location.Y > 24 then
				Object.Deleted := True;
			end if;
			Test_Hit(List, Object);
		end if;
	end;

	-- Shield

	procedure Initialize(Object: in out Shield) is
	begin
		Object.Width := 2;
		Object.Height := 1;
		Object.Life := 2;
	end;
	
	procedure Advance(Object: access Shield; List: in out Actor_List.List) is
	begin
		null;
	end;
	
	procedure Draw(Object: in out Shield; Screen: in out Game.Screen.Screen_Buffer) is
		S: array(1..2) of String(1..2) := ("", "  ");
	begin
		Game.Screen.Write(Screen, Object.Location.X, Object.Location.Y, S(Object.Life), 16#60#);
	end;

	procedure Collide(Object: in out Shield; Other: Actor_Access) is
	begin
		Object.Life := Object.Life - 1;
		if Object.Life = 0 then
			Object.Deleted := True;
		end if;
	end;

begin
	Random.Reset(Seed);
end Implementation;
